# coding: utf-8

from feiyan.http.fy_request import FyRequest
from feiyan.fy_client import FyClient


class Cloud:
    def __init__(self, app_key, app_secret):
        self.__app_key = app_key
        self.__app_secret = app_secret

    def __client(self):
        return FyClient(self.__app_key, self.__app_secret)

    def execute(self, url, params=None, token=None):
        req = FyRequest()
        req.set_url(url)
        req.set_cloud_token(token)
        req.set_params(params)
        return self.__client().execute(req)

    # 获取云端资源Token
    def token(self, params):
        url = '/cloud/token'
        return self.execute(url, params)

    # 刷新云端资源Token
    def refresh_token(self, token):
        url = '/cloud/token/refresh'
        params = {'cloudToken': token}
        return self.execute(url, params, token)

    # 查询物的产品列表
    def thing_product_list_get(self, params, token):
        url = '/cloud/thing/productList/get'
        return self.execute(url, params, token)

    # 查询物的产品
    def thing_product_get(self, params, token):
        url = '/cloud/thing/product/get'
        return self.execute(url, params, token)

    # 获取物的属性
    def thing_properties_get(self, params, token):
        url = '/cloud/thing/properties/get'
        return self.execute(url, params, token)

    # 获取物的模板
    def thing_tsl_get(self, params, token):
        url = '/cloud/thing/tsl/get'
        return self.execute(url, params, token)

    # 触发物的服务
    def thing_service_invoke(self, params, token):
        url = '/cloud/thing/service/invoke'
        return self.execute(url, params, token)

    # 设置物的属性
    def thing_properties_set(self, params, token):
        url = '/cloud/thing/properties/set'
        return self.execute(url, params, token)

    # 获取物的连接状态
    def thing_status_get(self, params, token):
        url = '/cloud/thing/status/get'
        return self.execute(url, params, token)

    # 获取物的基本信息
    def thing_info_get(self, params, token):
        url = '/cloud/thing/info/get'
        return self.execute(url, params, token)

    # 批量获取物
    def things_info_get(self, params, token):
        url = '/cloud/things/info/get'
        return self.execute(url, params, token)

    # 获取物的事件 timeline 数据
    def thing_event_timeline_get(self, params, token):
        url = '/cloud/thing/event/timeline/get'
        return self.execute(url, params, token)

    # 获取物的属性timeline数据
    def thing_property_timeline_get(self, params, token):
        url = '/cloud/thing/property/timeline/get'
        return self.execute(url, params, token)

    # 分页查询用户列表
    def account_query_identity_by_page(self, params, token):
        url = '/cloud/account/queryIdentityByPage'
        return self.execute(url, params, token)

    # 通过三方外标查询账号信息
    def account_get_by_openid(self, params, token):
        url = '/cloud/account/getByOpenId'
        return self.execute(url, params, token)

    # 获取用户绑定的设备列表（包括设备详情）详情内容
    def device_query_by_user(self, params, token):
        url = '/cloud/device/queryByUser'
        return self.execute(url, params, token)

    # 解绑用户和设备
    def user_device_unbind(self, params, token):
        url = '/cloud/user/device/unbind'
        return self.execute(url, params, token)



SYSTEM_HEADERS = (
    X_CA_SIGNATURE, X_CA_SIGNATURE_HEADERS, X_CA_TIMESTAMP, X_CA_NONCE, X_CA_KEY
) = (
    'X-Ca-Signature', 'X-Ca-Signature-Headers', 'X-Ca-Timestamp', 'X-Ca-Nonce', 'X-Ca-Key'
)

HTTP_HEADERS = (
    HTTP_HEADER_ACCEPT, HTTP_HEADER_CONTENT_MD5,
    HTTP_HEADER_CONTENT_TYPE, HTTP_HEADER_USER_AGENT, HTTP_HEADER_DATE
) = (
    'Accept', 'Content-MD5',
    'Content-Type', 'User-Agent', 'Date'
)

HTTP_PROTOCOL = (
    HTTP, HTTPS
) = (
    'http', 'https'
)

HTTP_METHOD = (
    GET, POST, PUT, DELETE, HEADER
) = (
    'GET', 'POST', 'PUT', 'DELETE', 'HEADER'
)

CONTENT_TYPE = (
    CONTENT_TYPE_FORM, CONTENT_TYPE_STREAM,
    CONTENT_TYPE_JSON, CONTENT_TYPE_XML, CONTENT_TYPE_TEXT
) = (
    'application/x-www-form-urlencoded', 'application/octet-stream',
    'application/json', 'application/xml', 'application/text'
)

BODY_TYPE = (
    FORM, STREAM
) = (
    'FORM', 'STREAM'
)






# coding: utf-8

import json
import time
from feiyan.http.request import Request


class FyRequest(Request):
    def __init__(self):
        Request.__init__(self)
        self.__params = dict()
        self.__cloudToken = None
        self.__version = '1.0'
        self.__apiVer = '1.0.0'

    def set_version(self, version):
        self.__version = version

    def get_version(self):
        return self.__version

    def set_api_ver(self, api_ver):
        self.__apiVer = api_ver

    def get_api_ver(self):
        return self.__apiVer

    def set_cloud_token(self, cloud_token):
        self.__cloudToken = cloud_token

    def get_cloud_token(self):
        return self.__cloudToken

    def add_param(self, k, v):
        self.__params[k] = v

    def set_params(self, params):
        self.__params = params

    def get_params(self):
        return self.__params

    def format_params(self):
        body = {
            "id": int(time.time() * pow(10, 6)),
            "version": self.get_version(),
            "request": {
                "apiVer": self.get_api_ver(),
                "cloudToken": self.get_cloud_token()
            },
            "params": self.get_params()
        }
        self.set_body(json.dumps(body))



# coding: utf-8

from feiyan import client
from feiyan.common import constant
from feiyan.http.fy_request import FyRequest


class FyClient:
    def __init__(self, app_key, app_secret):
        self.app_key = app_key
        self.app_secret = app_secret
        self.host = 'https://api.link.aliyun.com'

    def __client(self):
        return client.DefaultClient(app_key=self.app_key, app_secret=self.app_secret)

    def execute(self, fy_request):
        if not isinstance(fy_request, FyRequest):
            raise Exception('invalid fy_request instance')
        fy_request.set_host(self.host)
        fy_request.set_method('post')
        fy_request.set_protocol(constant.HTTPS)
        fy_request.set_content_type(constant.CONTENT_TYPE_JSON)
        fy_request.format_params()
        return self.__client().execute(fy_request)



from feiyan.common import constant


class Request:
    content_md5 = "Content-MD5"
    content_length = "Content-Length"
    content_type = "Content-Type"

    def __init__(self, host=None, protocol=constant.HTTP, headers=None, url=None, method=None, time_out=None):
        if headers is None:
            headers = {}
        self.__host = host
        self.__url = url
        self.__method = method
        self.__time_out = time_out
        self.__headers = headers
        self.__body = None
        self.__content_type = None
        self.__query_str = None
        self.__protocol = protocol

    def get_protocol(self):
        return self.__protocol

    def set_protocol(self, protocol):
        self.__protocol = protocol

    def get_method(self):
        return self.__method

    def set_method(self, method):
        self.__method = method.upper()

    def get_host(self):
        return self.__host

    def set_host(self, host):
        self.__host = host

    def get_url(self):
        return self.__url

    def set_url(self, url):
        self.__url = url

    def get_time_out(self):
        return self.__time_out

    def set_time_out(self, time_out):
        self.__time_out = time_out

    def get_content_type(self):
        return self.__content_type

    def set_content_type(self, content_type):
        self.__content_type = content_type

    def get_headers(self):
        return self.__headers

    def set_headers(self, headers=None):
        if headers is None:
            headers = {}
        self.__headers = headers

    def get_query_str(self):
        return self.__query_str

    def set_query_str(self, query_str=None):
        self.__query_str = query_str

    def set_body(self, body):
        self.__body = body

    def get_body(self):
        return self.__body



# coding: utf-8

import time
import uuid
import hmac
import hashlib
import base64
from feiyan.common import constant

TIME_ZONE = "GMT"
FORMAT_ISO_8601 = "%Y-%m-%dT%H:%M:%SZ"
FORMAT_RFC_2616 = "%a, %d %b %Y %X GMT"


def get_iso_8061_date():
    return time.strftime(FORMAT_ISO_8601, time.gmtime())


def get_rfc_2616_date():
    return time.strftime(FORMAT_RFC_2616, time.gmtime())


def get_timestamp():
    return str(int(time.time() * 1000))


def get_uuid():
    return str(uuid.uuid4())


def sign(source, secret):
    h = hmac.new(secret.encode('utf-8'), source.encode('utf-8'), hashlib.sha256)
    signature = base64.b64encode(h.digest()).strip().decode()
    return signature


def _get_md5(content):
    return hashlib.md5(content.encode('utf-8')).digest()


def get_md5_base64_str(content):
    return base64.b64encode(_get_md5(content)).strip().decode()


def build_sign_str(uri=None, method=None, headers=None, body=None):
    lf = '\n'
    string_to_sign = list()
    string_to_sign.append(method)

    string_to_sign.append(lf)
    if constant.HTTP_HEADER_ACCEPT in headers and headers[constant.HTTP_HEADER_ACCEPT]:
        string_to_sign.append(headers[constant.HTTP_HEADER_ACCEPT])

    string_to_sign.append(lf)
    if constant.HTTP_HEADER_CONTENT_MD5 in headers and headers[constant.HTTP_HEADER_CONTENT_MD5]:
        string_to_sign.append(headers[constant.HTTP_HEADER_CONTENT_MD5])

    string_to_sign.append(lf)
    if constant.HTTP_HEADER_CONTENT_TYPE in headers and headers[constant.HTTP_HEADER_CONTENT_TYPE]:
        string_to_sign.append(headers[constant.HTTP_HEADER_CONTENT_TYPE])

    string_to_sign.append(lf)
    if constant.HTTP_HEADER_DATE in headers and headers[constant.HTTP_HEADER_DATE]:
        string_to_sign.append(headers[constant.HTTP_HEADER_DATE])

    string_to_sign.append(lf)
    string_to_sign.append(_format_header(headers=headers))
    string_to_sign.append(_build_resource(uri=uri, body=body))

    return ''.join(string_to_sign)


def _build_resource(uri="", body=None):
    if body is None:
        body = {}
    if uri.__contains__("?"):
        uri_array = uri.split("?")
        uri = uri_array[0]
        query_str = uri_array[1]
        if not body:
            body = {}
        if query_str:
            query_str_array = query_str.split("&")
            for query in query_str_array:
                query_array = query.split("=")
                if query_array[0] not in body:
                    body[query_array[0]] = query_array[1]

    resource = list()
    resource.append(uri)
    if body:
        resource.append("?")
        param_list = list(body.keys())
        param_list.sort()
        first = True
        for key in param_list:
            if not first:
                resource.append("&")
            first = False

            if body[key]:
                resource.append(key)
                resource.append("=")
                resource.append(body[key])
            else:
                resource.append(key)

    if resource is None:
        return ''

    return "".join(str(x) for x in resource)


def convert_utf8(input_string):
    if isinstance(input_string, str):
        input_string = input_string.encode('utf-8')
    return input_string


def _format_header(headers=None):
    if headers is None:
        headers = {}
    lf = '\n'
    temp_headers = []
    if len(headers) > 0:
        header_list = list(headers.keys())
        header_list.sort()
        signature_headers = []
        for k in header_list:
            if k.startswith("X-Ca-"):
                temp_headers.append(k)
                temp_headers.append(":")
                temp_headers.append(str(headers[k]))
                temp_headers.append(lf)
                signature_headers.append(k)
        headers[constant.X_CA_SIGNATURE_HEADERS] = ','.join(signature_headers)
    return ''.join(temp_headers)


def device_data_sign(app_key, app_secret, message, msg_code):
    parameters = {
        'appKey': app_key,
        'message': message,
        'msgCode': msg_code
    }
    sign_str = _build_resource(body=parameters)[1:] + app_secret
    signature = hashlib.md5(sign_str.encode('utf-8')).hexdigest()
    return signature



# coding: utf-8

import requests
from feiyan.common import constant
from feiyan.auth.signature_composer import get_timestamp, get_uuid, get_md5_base64_str, build_sign_str, sign


class DefaultClient:
    def __init__(self, app_key=None, app_secret=None, time_out=None):
        self.__app_key = app_key
        self.__app_secret = app_secret
        self.__time_out = time_out
        pass

    def execute(self, request=None):
        try:
            headers = self.build_headers(request)
            r = requests.request(
                url=request.get_host() + request.get_url(),
                method=request.get_method(),
                headers=headers,
                data=request.get_body(),
                timeout=request.get_time_out()
            )
            return r
        except IOError:
            raise
        except AttributeError:
            raise

    def build_headers(self, request=None):
        headers = dict()
        header_params = request.get_headers()
        headers[constant.X_CA_TIMESTAMP] = get_timestamp()
        headers[constant.X_CA_KEY] = self.__app_key

        body = request.get_body()

        headers[constant.X_CA_NONCE] = get_uuid()

        if request.get_content_type():
            headers[constant.HTTP_HEADER_CONTENT_TYPE] = request.get_content_type()
        else:
            headers[constant.HTTP_HEADER_CONTENT_TYPE] = constant.CONTENT_TYPE_JSON

        if constant.HTTP_HEADER_ACCEPT in header_params \
                and header_params[constant.HTTP_HEADER_ACCEPT]:
            headers[constant.HTTP_HEADER_ACCEPT] = header_params[constant.HTTP_HEADER_ACCEPT]
        else:
            headers[constant.HTTP_HEADER_ACCEPT] = constant.CONTENT_TYPE_JSON

        if constant.POST == request.get_method() and constant.CONTENT_TYPE_JSON == request.get_content_type():
            headers[constant.HTTP_HEADER_CONTENT_MD5] = get_md5_base64_str(request.get_body())
            str_to_sign = build_sign_str(
                uri=request.get_url(),
                method=request.get_method(),
                headers=headers
            )
        else:
            str_to_sign = build_sign_str(
                uri=request.get_url(),
                method=request.get_method(),
                headers=headers, body=body
            )

        headers[constant.X_CA_SIGNATURE] = sign(str_to_sign, self.__app_secret)

        return headers



#!/usr/bin/env python
# coding:utf-8


from setuptools import setup, find_packages

with open("feiyan/README.md", "r") as f:
    long_description = f.read()

setup(
    name="feiyan_py3",
    version="1.0.1",
    author="jerrygaoyang",
    author_email="917616767@qq.com",
    description="iot feiyan package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jerrygaoyang/feiyan_py3",
    packages=find_packages(),
    install_requires=["requests"]
)
